import 'dart:io';

import 'package:cookie_jar/cookie_jar.dart';
import 'package:dio/adapter.dart';
import 'package:dio/dio.dart';
import 'package:dio_cookie_manager/dio_cookie_manager.dart';
import 'package:dio_http_cache/dio_http_cache.dart';
import 'package:yuro/util/assets.dart';

/// Dio构建器
class DioBuilder {
  Dio _dio = Dio();

  DioBuilder setBaseUrl(String url) {
    _dio.options.baseUrl = url;
    return this;
  }

  DioBuilder setContentType(String contentType) {
    _dio.options.contentType = contentType;
    return this;
  }

  DioBuilder setConnectTimeout(int timeout) {
    _dio.options.connectTimeout = timeout;
    return this;
  }

  DioBuilder setReceiveTimeout(int timeout) {
    _dio.options.receiveTimeout = timeout;
    return this;
  }

  DioBuilder addHeaders(Map<String, dynamic> headers) {
    if (headers != null && headers.isNotEmpty) {
      _dio.options.headers.addAll(headers);
    }
    return this;
  }

  DioBuilder addQueryParameters(Map<String, dynamic> parameters) {
    if (parameters != null && parameters.isNotEmpty) {
      _dio.options.queryParameters.addAll(parameters);
    }
    return this;
  }

  DioBuilder addExtras(Map<String, dynamic> extras) {
    if (extras != null && extras.isNotEmpty) {
      _dio.options.extra.addAll(extras);
    }
    return this;
  }

  DioBuilder addCookManager(CookieJar cookieJar) {
    _dio.interceptors.add(CookieManager(cookieJar ?? DefaultCookieJar()));
    return this;
  }

  DioBuilder addInterceptors(List<Interceptor> interceptors) {
    _dio.interceptors.addAll(interceptors);
    return this;
  }

  DioBuilder certificate(String pem) {
    (_dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate = (client) {
      client.badCertificateCallback = (cert, host, port) => cert.pem == pem;
    };
    return this;
  }

  DioBuilder certificateFile(File file) {
    (_dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate = (client) {
      return HttpClient(context: SecurityContext()..setTrustedCertificates(file.path));
    };
    return this;
  }

  /// 开启缓存
  DioBuilder openCache() {
    _dio.interceptors.add(DioCacheManager(CacheConfig(baseUrl: _dio.options.baseUrl)).interceptor);
    return this;
  }

  /// 开启日志打印
  DioBuilder openLog({
    bool request,
    bool requestHeader,
    bool requestBody,
    bool responseHeader,
    bool responseBody,
  }) {
    _dio.interceptors.add(LogInterceptor(
        request: request ?? true,
        requestHeader: requestHeader ?? true,
        requestBody: requestBody ?? false,
        responseHeader: responseHeader ?? true,
        responseBody: responseBody ?? false));
    return this;
  }

  /// mock数据
  DioBuilder openMock(bool open) {
    if (open) {
      _dio.interceptors.add(MockInterceptor(_dio));
    }
    return this;
  }

  ///@param proxy 代理地址及端口 eg. 192.168.1.1:8888
  DioBuilder openProxy(String proxy) {
    (_dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate = (client) {
      if (proxy != null) {
        client.findProxy = (uri) => "PROXY $proxy";
//        client.badCertificateCallback = (cert, host, port) => true;
      }
    };
    return this;
  }

  Dio build() => _dio;
}

/// mock数据拦截器
///
/// @note assets目录下创建mock文件夹,并在[pubspec.yaml]中注册
///
/// @note 文件名为请求路径替换'/'为'_',文件后缀为.json
class MockInterceptor implements Interceptor {
  final Dio dio;

  MockInterceptor(this.dio);

  @override
  Future onRequest(RequestOptions options) async {
    String action = options.path.replaceAll('/', '_');
    var response = await AssetsUtil.loadJson(action, path: 'assets/mock');
    return dio.resolve(response);
  }

  @override
  Future onResponse(Response response) async => response;

  @override
  Future onError(DioError err) async => err;
}

/// 错误拦截器
class ErrorInterceptor implements Interceptor {
  @override
  Future onRequest(RequestOptions options) async => options;

  @override
  Future onResponse(Response response) async => response;

  @override
  Future onError(DioError err) async {
    err.error = DioException.create(err);
    return err;
  }
}

/// DioError转换
class DioException implements Exception {
  final int code;
  final String message;

  DioException([this.code, this.message]);

  String toString() => '$code,$message';

  factory DioException.create(DioError error) {
    switch (error.type) {
      case DioErrorType.CONNECT_TIMEOUT:
        return DioException(-1, '连接超时');
      case DioErrorType.SEND_TIMEOUT:
        return DioException(-1, '请求超时');
      case DioErrorType.CANCEL:
        return DioException(-1, '请求取消');
      case DioErrorType.RECEIVE_TIMEOUT:
        return DioException(-1, '响应超时');
      case DioErrorType.RESPONSE:
        return DioException(error.response.statusCode, error.response.statusMessage);
      default:
        switch (error.error.runtimeType.toString()) {
          case 'SocketException':
            return parseSocketException(error.error as SocketException);
            break;
          case 'FormatException':
            var e = error.error as FormatException;
            return DioException(-1, e.message);
            break;
          default:
            return DioException(-1, error.message);
        }
    }
  }

  static DioException parseSocketException(SocketException exception) {
    switch (exception.osError.errorCode) {
      case 101:
        return DioException(101, '网络不可用');
      case 111:
        return DioException(111, '服务器不可用');
      default:
        return DioException(exception.osError.errorCode, exception.osError.message);
    }
  }
}
